import 'package:flutter/material.dart';
import 'package:flutter_map/flutter_map.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';
import 'package:latlong2/latlong.dart';
import 'package:gui_app/provider.dart';
import 'package:gui_app/const/colors.dart';
import 'package:gui_app/const/config.dart';
import 'package:gui_app/const/dimens.dart';

class OfflineMap extends ConsumerStatefulWidget {
  const OfflineMap({super.key});
  @override
  ConsumerState<OfflineMap> createState() {
    return _MapState();
  }
}

class _MapState extends ConsumerState<OfflineMap>
    with TickerProviderStateMixin {
  final _mapController = MapController();
  late Map<String, double> pinSize;
  late Map<String, double> latitude;
  late Map<String, double> longitude;

  @override
  void initState() {
    super.initState();
    pinSize = {
      'Sapporo': mapPinDefaultSize,
      'Tokyo': mapPinDefaultSize,
      'Osaka': mapPinDefaultSize,
      'Asahikawa': mapPinDefaultSize,
      'Fukushima': mapPinDefaultSize,
    };
    latitude = initLatitude();
    longitude = initLongitude();
  }

  Map<String, double> initLatitude() {
    Map<String, double> initValue = {
      'Sapporo': mapCenterLatitude,
      'Tokyo': mapTokyoLatitude,
      'Osaka': mapOsakaLatitude,
      'Asahikawa': mapAsahikawaLatitude,
      'Fukushima': mapFukushimaLatitude,
    };
    return initValue;
  }

  Map<String, double> initLongitude() {
    Map<String, double> initValue = {
      'Sapporo': mapCenterLongitude,
      'Tokyo': mapTokyoLongitude,
      'Osaka': mapOsakaLongitude,
      'Asahikawa': mapAsahikawaLongitude,
      'Fukushima': mapFukushimaLongitude,
    };
    return initValue;
  }

  void _animatedZoom(LatLng destLocation, double zoom) {
    final latTween = Tween<double>(
        begin: _mapController.center.latitude, end: destLocation.latitude);
    final lngTween = Tween<double>(
        begin: _mapController.center.longitude, end: destLocation.longitude);
    final zoomTween = Tween(begin: _mapController.zoom, end: zoom);
    final controller = AnimationController(
        duration: const Duration(milliseconds: mapAnimationDurationMs),
        vsync: this);
    final Animation<double> animation =
        CurvedAnimation(parent: controller, curve: Curves.fastOutSlowIn);

    controller.addListener(() {
      _mapController.move(
          LatLng(latTween.evaluate(animation), lngTween.evaluate(animation)),
          zoomTween.evaluate(animation));
    });

    animation.addStatusListener((status) {
      if (status == AnimationStatus.completed) {
        controller.dispose();
      } else if (status == AnimationStatus.dismissed) {
        controller.dispose();
      }
    });

    controller.forward();
  }

  Marker _createMarker(String label) {
    return Marker(
      point: LatLng(latitude[label]!, longitude[label]!),
      builder: (ctx) => MouseRegion(
        cursor: SystemMouseCursors.click,
        child: GestureDetector(
          onTap: () {
            setState(() {
              pinSize.updateAll((key, value) => mapPinDefaultSize);
              pinSize[label] = mapPinFocusSize;
              // Icon is aligned in the upper left corner,
              // so if you increase the size, it will shift.
              latitude = initLatitude();
              longitude = initLongitude();
              latitude[label] = latitude[label]! + mapPinLatitudeSlide;
              longitude[label] = longitude[label]! + mapPinLongitudeSlide;
            });
          },
          child: Icon(
            Icons.location_pin,
            color: mapPointColor,
            size: pinSize[label],
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final Size containerSize = MediaQuery.of(context).size;
    return Scaffold(
      body: Stack(
        alignment: Alignment.bottomRight,
        children: [
          FlutterMap(
            mapController: _mapController,
            options: MapOptions(
                center: LatLng(mapCenterLatitude, mapCenterLongitude),
                zoom: mapMinZoom,
                maxZoom: mapMaxZoom,
                minZoom: mapMinZoom,
                swPanBoundary: LatLng(mapMinLatitude, mapMinLongitude),
                nePanBoundary: LatLng(mapMaxLatitude, mapMaxLongitude),
                onLongPress: (tapPosition, point) {
                  if (_mapController.zoom != mapMinZoom)
                    _animatedZoom(point, mapMinZoom);
                }),
            layers: [
              TileLayerOptions(
                tileProvider: AssetTileProvider(),
                urlTemplate: 'assets/map/{z}/{x}/{y}.png',
              ),
              MarkerLayerOptions(
                markers: [
                  _createMarker('Sapporo'),
                  _createMarker('Tokyo'),
                  _createMarker('Osaka'),
                  _createMarker(
                    'Asahikawa',
                  ),
                  _createMarker(
                    'Fukushima',
                  ),
                ],
              ),
            ],
          ),
          Text(
            "\u00a9 https://www.openstreetmap.org/copyright contributors",
            style: TextStyle(
                fontSize: containerSize.width * mapCopyrightFontSizeRatio),
          ),
        ],
      ),
    );
  }
}
